/**
 * PETALS - PETALS Services Platform. Copyright (c) 2005 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */

package org.ow2.petals.jmx;

import java.io.IOException;
import java.util.Hashtable;
import java.util.Set;

import javax.jbi.management.AdminServiceMBean;
import javax.management.MBeanServerConnection;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;

import org.ow2.petals.jmx.exception.AdminDoesNotExistException;
import org.ow2.petals.jmx.exception.AdminServiceErrorException;
import org.ow2.petals.jmx.exception.PerformActionErrorException;

/**
 * Client API of the JMX Admin service.
 * 
 * @author rnaudin - EBM Websourcing
 * 
 */
public class AdminServiceClient extends AbstractServiceClient {

    /**
     * Action : get System Info
     */
    private static final String GET_SYSTEM_INFO = "getSystemInfo";

    /**
     * Action : get Binding Components
     */
    private static final String GET_BINDING_COMPONENTS = "getBindingComponents";

    /**
     * Action : get Services Engines
     */
    private static final String GET_ENGINE_COMPONENTS = "getEngineComponents";

    /**
     * Action : get Component By Name
     */
    private static final String GET_COMPONENT_BY_NAME = "getComponentByName";

    private final static String ADMIN_SERVICE_JMX_NAME = "Admin";

    /**
     * Creates a new instance of {@link AdminServiceClient}
     * 
     * @param petalsDomain
     * @param beanServerConnection
     * @throws AdminDoesNotExistException
     * @throws AdminServiceErrorException
     */
    public AdminServiceClient(String petalsDomain, MBeanServerConnection beanServerConnection)
            throws AdminDoesNotExistException, AdminServiceErrorException {
        super(petalsDomain, beanServerConnection);

        try {
            Set<ObjectName> objNames;

            Hashtable<String, String> attributes = new Hashtable<String, String>();
            attributes.put(JMX_FIELD_NAME, ADMIN_SERVICE_JMX_NAME);
            attributes.put(JMX_FIELD_TYPE, JMX_SERVICE_TYPE);
            ObjectName objName = new ObjectName(petalsDomain, attributes);
            objNames = this.mBeanServerConnection.queryNames(objName, null);
            if (objNames != null && objNames.size() == 1) {
                this.mbeanName = objNames.iterator().next();
            } else {
                throw new AdminDoesNotExistException();
            }
        } catch (MalformedObjectNameException e) {
            throw new AdminServiceErrorException(e);
        } catch (IOException e) {
            throw new AdminServiceErrorException(e);
        }
    }

    /**
     * Get the version of the kernel of the PEtALS container
     * 
     * @see AdminServiceMBean#getSystemInfo()
     * @return the version of PEtALS.
     * @throws PerformActionErrorException
     */
    public String getSystemInfo() throws AdminServiceErrorException {
        String result;

        try {
            result = (String) this.performAction(GET_SYSTEM_INFO, null, null);
        } catch (PerformActionErrorException e) {
            throw new AdminServiceErrorException(e);
        } catch (ClassCastException e) {
            throw new AdminServiceErrorException("Unexpected result type");
        }

        return result;
    }

    /**
     * Get the Binding Components of the PEtALS Container
     * 
     * @see AdminServiceMBean#getBindingComponents()
     * @return the binding components of the PEtALS Container
     * @throws AdminServiceErrorException
     */
    public ObjectName[] getBindingComponents() throws AdminServiceErrorException {
        ObjectName[] components;

        try {
            components = (ObjectName[]) this.performAction(GET_BINDING_COMPONENTS, null, null);
        } catch (PerformActionErrorException e) {
            throw new AdminServiceErrorException(e);
        } catch (ClassCastException e) {
            throw new AdminServiceErrorException("Unexpected result type");
        }

        return components;
    }

    /**
     * Get the Services Engines of the PEtALS Container
     * 
     * @return the services engines of the PEtALS Container
     * @throws AdminServiceErrorException
     */
    public ObjectName[] getEngineComponents() throws AdminServiceErrorException {
        ObjectName[] components;

        try {
            components = (ObjectName[]) this.performAction(GET_ENGINE_COMPONENTS, null, null);
        } catch (PerformActionErrorException e) {
            throw new AdminServiceErrorException(e);
        } catch (ClassCastException e) {
            throw new AdminServiceErrorException("Unexpected result type");
        }

        return components;
    }

    /**
     * Get the Component with the given name
     * 
     * @return null if no component has been found, the {@link ObjectName} of
     *         the component otherwise
     * @throws AdminServiceErrorException
     *             if an error occurs while performing the action
     */
    public ObjectName getComponentByName(String component) throws AdminServiceErrorException {
        ObjectName componentName;

        Object[] objectsLoad = new Object[] { component };
        String[] stringsLoad = new String[] { String.class.getName() };
        try {
            componentName = (ObjectName) this.performAction(GET_COMPONENT_BY_NAME, objectsLoad,
                    stringsLoad);
        } catch (PerformActionErrorException e) {
            throw new AdminServiceErrorException(e);
        } catch (ClassCastException e) {
            throw new AdminServiceErrorException("Unexpected result type");
        }

        return componentName;
    }
}
